#include "spamtac.h"
#include "game.h"
#include "title.h"
#include "timer.h"

DATAFILE *data;
int num_players, time_limit;
BITMAP *mouse_target;

static void intro_screen(void) {
	BITMAP *bmp;

	// Miquel Fire logo
	clear_bitmap(screen);
	set_palette(black_palette);
	bmp = (BITMAP*)data[MF_LOGO].dat;
	blit(bmp, screen, 0, 0, 400-bmp->w/2, 300-bmp->h/2, bmp->w, bmp->h);
	fade_in((RGB*)data[MF_PAL].dat, 3);
	play_sample((SAMPLE*)data[MIQUELFIRE_WAV].dat, 255, 128, 1000, 0);
	rest(1200);
	fade_out(3);
	
	// SpeedHack logo
	clear_bitmap(screen);
	bmp = (BITMAP*)data[SH06_BMP].dat;
	stretch_blit(bmp, screen, 0, 0, bmp->w, bmp->h, 0, 0, 800, 600);
	fade_in((RGB*)data[SH06_PAL].dat, 3);
	play_sample((SAMPLE*)data[SPEEDHACK_WAV].dat, 255, 128, 1000, 0);
	rest(1600);
	fade_out(3);
}

static void init_data(void) {
	// Load datafile, set mouse cursor
	char buf[256], buf2[256];
	get_executable_name(buf, sizeof(buf));
	replace_filename(buf2, buf, "media/data.dat", sizeof(buf2));
	set_color_conversion(COLORCONV_NONE);
	data = load_datafile(buf2);
	if (!data) {
		set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
		allegro_message("Error loading %s\n", buf2);
		exit(1);
	}

	mouse_target = create_bitmap(16, 16);
	clear_bitmap(mouse_target);
	circle(mouse_target, 7, 7, 6, 1);
	hline(mouse_target, 0, 7, 2, 1);
	hline(mouse_target, 12, 7, 14, 1);
	vline(mouse_target, 7, 0, 2, 1);
	vline(mouse_target, 7, 12, 14, 1);

	set_mouse_sprite(mouse_target);
	set_mouse_sprite_focus(7, 7);
	show_mouse(screen);
	default_palette[1].r = default_palette[1].g = default_palette[1].b = 63;
}

static void clean_up_data(void) {
	destroy_bitmap(mouse_target);
	unload_datafile(data);
}

int main() {
	Timer::Intialize();
	set_uformat(U_ASCII_CP);
	srand(time(NULL));
	if (allegro_init() != 0)
		return 1;
	install_keyboard(); // Really needed?
	install_timer();
	install_mouse();

	if (install_sound(DIGI_AUTODETECT, MIDI_AUTODETECT, 0) != 0) {
		allegro_message("Error setting up the sound!\n%s\n", allegro_error);
		install_sound(DIGI_NONE, MIDI_NONE, 0);
	}

	if (set_gfx_mode(GFX_AUTODETECT, 800, 600, 0, 0) != 0) {
		set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
		allegro_message("Error setting up a 800x600x8 display\n%s\n", allegro_error);
		return 1;
	}

	init_data();

	rest(500); // Give slow switching screen a chance to catch up
	intro_screen();

	clear_bitmap(screen);

	while (title_screen())
		play_game();

	clean_up_data();
	set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
	allegro_exit();

	return 0;
}
END_OF_MAIN()
